# -*- coding: utf8 -*-
import urllib
import os
import shutil
import time

import config
import utils 
from Tag import Tag
from TagManager import TagManager
import Request
import Category
import Repository
import Permission

from WikiException import WikiException
class ExistPage(WikiException): pass 
class ExistCategory(WikiException): pass 
class EditConflict(WikiException): pass
class NotExistPage(WikiException): pass
class InvalidPageName(WikiException): pass
class NotExistAttachment(WikiException): pass

def createPage(category, pageName, rawText, tags='', perms=''):
	if not utils.isValidName(pageName):
		raise InvalidPageName
	encPageName = utils.encode(pageName)
	filePath = os.path.join(category.dirPath, encPageName)
	if os.path.isfile(filePath):
		raise ExistPage
	if os.path.isdir(filePath):
		raise ExistCategory
	tm = TagManager(tags)
	tags = tm.getTagString()
	if not perms:
		perms = category.perms
	file(filePath, 'wb').write('%s\n%s\n%s' % (perms, tags, rawText))
	page = Page(category, pageName)
	for tag in tm.toSet():
		Tag(tag).append(page)
	return page

class Page(Permission.PermissionChecker):
	def __init__(self, category, pageName):
		self.category = category
		self.pageName = pageName
		self.encPageName = utils.encode(pageName)
		self._setPagePath() # self.pagePath
		self._setFilePath() # self.filePath
		self.load() # self.rawText, self.tags, self.ctime

	def _setPagePath(self):
		self.pagePath = utils.joinPath(self.category.categoryPath, self.pageName)

	def _setFilePath(self):
		self.filePath = os.path.join(self.category.dirPath, self.encPageName)
		self.attachmentFolderPath = os.path.join(Repository.getAttachmentRepository(), utils.encode(self.pagePath))

	def load(self):
		fp = file(self.filePath, 'rb')
		self.perms = Permission.Permission(fp.readline().strip())
		self.tags = fp.readline().strip()
		self.rawText = fp.read()
		fp.close()
		self._setMTime()

	def setRawText(self, rawText):
		self.rawText = rawText
	def setTags(self, tags):
		self.tags = tags.strip()
	def _save(self):
		fp = file(self.filePath, 'rb')
		orgTags = fp.readline().strip()
		fp.close()

		tm = TagManager(orgTags)
		tm.update(self.tags, self)

		file(self.filePath, 'wb').write('%s\n%s\n%s' % (self.perms, self.tags, self.rawText))
		self._setMTime()

	def _setMTime(self):
		self.mtime = str(os.path.getmtime(self.filePath))

	def edit(self, rawText, tags, perms=''):
		if not perms:
			perms = self.perms
		self.perms = Permission.Permission(perms)
		self.rawText = rawText
		oldTags = self.tags
		self.tags = tags.strip()
		TagManager(oldTags).update(self.tags, self)
		self._save()

	def editPerms(self, s): 
		self.perms = Permission.Permission(s.strip())
		self._save()
	def getGroup(self):
		return self.category.getGroup()
	def backup(self, timeStamp=None):
		backupRepository = Repository.getPageBackupRepository()
		backupDir = os.path.join(backupRepository, utils.encode(self.category.categoryPath))
		if not os.path.exists(backupDir):
			os.makedirs(backupDir)
		if not timeStamp:
			timeStamp = int(time.time()) # FIXME int?
		descPath = '%s.%s' % (os.path.join(backupDir, self.encPageName), timeStamp)
		shutil.copy2(self.filePath, descPath)

	def remove(self):
		if not os.path.isfile(self.filePath):
			raise NotExistPage
		# FIXME TagManager에 remove를 추가하고 이부분이랑 교체
		tm = TagManager(self.tags)
		tm.update('', self)
		# ^^^^^^^^^^^^^^^^^^^^^^^^^
		os.remove(self.filePath)
		if self.category.isEmpty():
			self.category.remove()
		return True

	def toLink(self, klass='', linkName=''):
		return utils.makeLink(self.category.categoryPath, self.pageName, klass=klass, linkName=linkName)
	def createAttachmentFolder(self):
		if os.path.exists(self.attachmentFolderPath):
			return False
		os.makedirs(self.attachmentFolderPath)
		return True
	def isExistAttachmentFolder(self): #FIXME name...
		if os.path.isdir(self.attachmentFolderPath):
			return True
		return False
	def attach(self, fileName, fileValue):
		if not self.isExistAttachmentFolder():
			if not self.createAttachmentFolder(): # FIXME 아예 raise를..?
				return None
		fileName = fileName.replace(' ', '_')
		filePath = os.path.join(self.attachmentFolderPath, fileName)
		while os.path.exists(filePath):
			name, ext = os.path.splitext(filePath)
			filePath = name + '_' + ext
		file(filePath, 'wb').write(fileValue)
		return filePath.split('/')[-1]
	def isExistAttachment(self, fileName): # FIXME 테스트코드 없음.
		filePath = os.path.join(self.attachmentFolderPath, fileName)
		if os.path.exists(filePath):
			return True
		return False
	def getAttachedFiles(self):
		if self.isExistAttachmentFolder():
			return os.walk(self.attachmentFolderPath).next()[2]
		return []
	def getAttachment(self, fileName):
		filePath = os.path.join(self.attachmentFolderPath, fileName)
		if not os.path.exists(filePath):
			return None # FIXME or raise
		return file(filePath, 'rb').read()
		
	def backupAttachment(self, fileName, timeStamp):
		if not self.isExistAttachment(fileName):
			raise NotExistAttachment
		backupRepository = Repository.getAttachmentBackupRepository()
		backupDir = os.path.join(backupRepository, utils.encode(self.pagePath))
		if not os.path.exists(backupDir):
			os.makedirs(backupDir)
		srcPath = os.path.join(self.attachmentFolderPath, fileName)
		newFileName = '%s.%s' % (fileName, timeStamp)
		descPath = os.path.join(backupDir, newFileName)
		shutil.move(srcPath, descPath)# FIXME copy2?
		return newFileName

	def backupAll(self, timeStamp): # FIXME testcode 없음
		self.backup(timeStamp)
		for fileName in self.getAttachedFiles():
			self.backupAttachment(fileName, timeStamp)

	def removeAttachment(self, fileName): # FIXME 사용되지 않음.
		if not self.isExistAttachment(fileName):
			raise NotExistAttachment
		attachmentPath = os.path.join(self.attachmentFolderPath, fileName)
		os.remove(attachmentPath)

def getPage(categoryPath, pageName):
	category = Category.getCategoryByCategoryPath(categoryPath)
	if category:
		page = category.getPage(pageName) # FIXME category에서 FrontPage로 얻어올 수 있음.
		return page
	return None

def getPageFromDescendants(categoryPath, pageName):
	category = Category.getCategoryByCategoryPath(categoryPath)
	if not category:
		return None
	return category.getPageFromDescendants(pageName)

def travelAlongRails(categoryPath, pageName):
	page = getPageFromDescendants(categoryPath, pageName)
	if page:
		return page
	category = Category.getCategoryByCategoryPath(utils.joinPath(categoryPath, pageName))
	if category:
		pageName = 'FrontPage'
	page = getPage(config.globalCategoryName, pageName)
	if page:
		return page
	return None
